@@ -55,6 +55,7 @@
 #include "ip_info.h"
 #include "lib/crc32.h"
 #include "lib/log.h"
+#include "lib/trim.h"
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
@@ -552,6 +553,33 @@ static int ide_key_is_cloud_id()
 	return 1;
 }
 
+static bool is_opcache_enabled()
+{
+	zend_string *opcache_enable = ZSTR_INIT_LITERAL("opcache.enable", 0);
+	zend_string *opcache_enable_cli = ZSTR_INIT_LITERAL("opcache.enable_cli", 0);
+	zend_string *opcache_optimization_level = ZSTR_INIT_LITERAL("opcache.optimization_level", 0);
+
+	zend_string *opcache_enable_v = zend_ini_get_value(opcache_enable);
+	zend_string *opcache_enable_cli_v = zend_ini_get_value(opcache_enable_cli);
+	zend_string *opcache_optimization_level_v = zend_ini_get_value(opcache_optimization_level);
+
+	zend_string_release(opcache_enable);
+	zend_string_release(opcache_enable_cli);
+	zend_string_release(opcache_optimization_level);
+
+	if (!opcache_enable_v || zend_string_equals_literal(opcache_enable_v, "0")) {
+		return false;
+	}
+	if (!opcache_enable_cli_v || zend_string_equals_literal(opcache_enable_cli_v, "0")) {
+		return false;
+	}
+	if (!opcache_optimization_level_v || zend_string_equals_literal(opcache_optimization_level_v, "0")) {
+		return false;
+	}
+
+	return true;
+}
+
 static void warn_if_opcache_is_loaded_after_xdebug()
 {
 	bool xdebug_loaded = false;
@@ -565,7 +593,7 @@ static void warn_if_opcache_is_loaded_after_xdebug()
 			xdebug_loaded = true;
 		}
 
-		if (strcmp(zext->name, "Zend OPcache") == 0) {
+		if (strcmp(zext->name, "Zend OPcache") == 0 && is_opcache_enabled()) {
 			if (xdebug_loaded) {
 				xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "OPCACHE", "Debugger is not working optimally, as Xdebug is loaded before Zend OPcache");
 			}
